/* Copyright (C) 2014 Dan Chapman <dpniel@ubuntu.com>

   This file is part of Dekko email client for Ubuntu Touch/

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import QtQuick 2.2
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 1.0 as ListItem
import Ubuntu.Components.Popups 1.0
import TrojitaCore 0.1
import DekkoCore 0.1
import "../Components"


BasePageWithComposer {
    id: mailboxListPage

    pageTitle: isNestedSomewhere ? currentMailbox : i18n.tr("Mailbox")
    tipText: i18n.tr("Compose New")
    // As this will never be a reply message we set it
    // to REPLY_NONE to maintain an empty composer
    replyMode: ReplyMode.REPLY_NONE
    flickable: null
//    head.backAction: backAction
//    head.actions: [openAccountsAction, connectionDialogAction]

    property QtObject model
    property int nestingDepth: 0
    property string currentMailbox
    property string currentMailboxLong
    property bool isNestedSomewhere: nestingDepth > 0
    readonly property alias isLoading: internal.isLoading
    property alias developerModeEnabled: settingsAction.visible

    signal mailboxSelected(string mailbox)

    function goUpOneLevel() {
        if (nestingDepth > 0) {
            internal.openParentMailbox()
        } else {
            console.log("[LOG] Already at the root of the mailbox tree. Cannot go up any more levels")
        }
    }

    QtObject {
        id: internal

        property bool isLoading: imapAccess.visibleTasksModel.hasVisibleTasks

        onIsLoadingChanged: handleMaybeLoading()

        function openParentMailbox() {
            model.setRootOneLevelUp()
            --nestingDepth
            currentMailbox = imapAccess.mailboxListShortMailboxName()
            currentMailboxLong = imapAccess.mailboxListMailboxName()
            title = nestingDepth ? imapAccess.mailboxListShortMailboxName() : i18n.tr("Mailbox")
        }

        function openChildMailbox(index, shortName, longName) {
            currentMailbox = shortName
            currentMailboxLong = longName
            ++nestingDepth
            model.setRootItemByOffset(index)
            title = shortName
        }

        function isNestedSomewhere() {
            return nestingDepth > 0
        }

        function handleMaybeLoading() {
            // In other words if *some* task is currently running
            // and we are the only page on the stack then show the popover
            // The message list page has it's own task progressbar
            if (isLoading && pageStack.depth === 1) {
                PopupUtils.open(loadingOverlay)
            }
        }
    }

    Action {
        id: backAction
        iconName: "back"
        text: i18n.tr("back")
        onTriggered: goUpOneLevel()
        visible: bottomEdgeOpened ? false : isNestedSomewhere
    }

    Action {
        id: openAccountsAction
        iconName: "contact-group"
        text: i18n.tr("Accounts")
        onTriggered: dekko.showAccounts()
    }

    Action {
        id: connectionDialogAction
        iconName: "language-chooser"
        text: i18n.tr("Network")
        onTriggered: {
            PopupUtils.open(Qt.resolvedUrl("../Dialogs/NetworkPolicyDialog.qml"))
        }
    }

    Action {
        id: settingsAction
        iconName: "settings"
        text: i18n.tr("Settings")
        onTriggered: {
            pageStack.clear()
            pageStack.push(Qt.resolvedUrl("../DeveloperMode/DevSettingsTabs.qml"))
        }

    }

    MailboxListView {
        id: mboxListView
        anchors.fill: parent
        listModel: mailboxListPage.model
        onOpenMailbox: {
            currentMailbox = shortMboxName
            currentMailboxLong = mboxName
            mailboxSelected(mboxName)
        }
        onOpenChildMailbox: internal.openChildMailbox(index, shortMboxName, mboxName)
    }

    Component {
        id: loadingOverlay
        LoadingOverlay {}
    }

    pageHead: PageHeadConfiguration {
        backAction: backAction
        actions: [
            openAccountsAction,
            connectionDialogAction,
            settingsAction
        ]
    }
}
