/* Copyright (C) 2014 Dan Chapman <dpniel@ubuntu.com>

   This file is part of Dekko email client for Ubuntu Devices/

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import QtQuick 2.2
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 1.0 as ListItem
import Ubuntu.Components.Popups 1.0

Column {
    id: attachmentView
    property string fileName
    property string mimeDescription
    property string fileSize

    property var previewWidget
    property bool showPreview: false

    property string downloadFilePath

    property bool isAttachmentView: true

    signal open
    signal openComplete

    width: parent.width

    ListItem.Subtitled {
        id: attachmentItem
        iconName: "empty-symbolic"
        iconFrame: false
        text: fileName
        subText: mimeDescription + ", " + fileSize
        onClicked: {
            PopupUtils.open(contextMenu, attachmentItem)
        }
        Component {
            id: contextMenu
            Popover {
                id: popover
                Column {
                    id: containerLayout
                    anchors {
                        left: parent.left
                        top: parent.top
                        right: parent.right
                    }
                    ListItem.Standard {
                        visible: previewWidget !== undefined
                        text: i18n.tr("Show Preview")
                        control: CheckBox {
                            checked: previewWidget.visible
                            onCheckedChanged: {
                                showPreview = checked
                                previewWidget.visible = checked;
                                PopupUtils.close(popover)
                            }
                        }
                    }
                    ListItem.Standard {
                        text: i18n.tr("Open")
                        onClicked: {
                            open();
                            PopupUtils.close(popover)
                        }
                    }
                }
            }
        }
    }

    function openDownloadComplete(path) {
        downloadFilePath = path;
        var contentDialog = PopupUtils.open(Qt.resolvedUrl("../Dialogs/ContentPickerDialog.qml"));
        contentDialog.complete.connect(function() {
            attachmentView.openComplete();
        });
    }
}
