import QtQuick 2.0
import Ubuntu.Components 1.1
import Ubuntu.Components.Popups 1.0
import TrojitaCore 0.1
import DekkoCore 0.1

Page {
    id: addNewAccountPage
    title: i18n.tr("Auto Configuration");

    property alias userName : userNameInput.text
    property alias password: passwordInput.text

    head {
        backAction: Action {
            enabled: imapAccess.sslMode != "";
            iconName: "close"
            onTriggered: {
                pageStack.pop();
            }
        }
        actions: [
            Action {
                iconName: "select"
                onTriggered: {
                    mailConfig.config(userNameInput.text)
                    Qt.inputMethod.hide()
                    PopupUtils.open(configuringDialogComponent)
                }
            }
        ]
    }

    Component {
        id: configuringDialogComponent
        Dialog {
            id: configuringDialog

            property string _mailAutoConfigStatus: "INPROGRESS" // "INPROGRESS", "ERROR"
            property string _mailAutoConfigErrorMessage

            title: "Looking for Configuration"
            text: configuringDialog._mailAutoConfigStatus === "ERROR" ?
                      configuringDialog._mailAutoConfigErrorMessage : ""
            ActivityIndicator {
                height: units.gu(4)
                width: height
                visible: running
                running: configuringDialog._mailAutoConfigStatus == "INPROGRESS"
            }
            Button {
                text: i18n.tr("Try again")
                visible: configuringDialog._mailAutoConfigStatus == "ERROR"
                onClicked: PopupUtils.close(configuringDialog)
            }
            Button {
                id: manualConfigButton
                text: i18n.tr("Configure Manually")
                visible: configuringDialog._mailAutoConfigStatus == "ERROR"
                onClicked: {
                    dekko.manualConfig(addNewAccountPage.userName, addNewAccountPage.password)
                    PopupUtils.close(configuringDialog)
                }
            }
            Connections {
                target: mailConfig
                onComplete: {
                    dekko.autoConfigComplete(config, addNewAccountPage.userName, addNewAccountPage.password);
                    PopupUtils.close(configuringDialog);
                }
                onFailed: {
                    configuringDialog._mailAutoConfigStatus = "ERROR";
                    configuringDialog._mailAutoConfigErrorMessage =
                            "Cannot find configuration for " + addNewAccountPage.userName + ".";
                }
            }
        }
    }

    Flickable {
        width: parent.width
        height: parent.height
        clip: true
        contentHeight: col.height + units.gu(5)

        Column {
            id: col
            spacing: units.gu(1)
            anchors {
                top: parent.top
                left: parent.left
                right: parent.right
                margins: units.gu(2)
            }
            Label {
                id: infoLabel
                width: parent.width
                anchors.bottomMargin: units.gu(2)
                wrapMode: TextEdit.Wrap
                text: i18n.tr("Enter your email address and Dekko will attempt to retrieve your account settings.")
            }

            Label {
                id: usernameLabel
                property bool valid: !!userNameInput.text
                text: valid ? i18n.tr("Email address") : i18n.tr("Email address required")
                color: valid ? "#888888" : "red"
            }
            TextField {
                id: userNameInput
                anchors {
                    left: col.left;
                    right: col.right;
                }
                inputMethodHints: Qt.ImhNoAutoUppercase | Qt.ImhEmailCharactersOnly | Qt.ImhNoPredictiveText
                KeyNavigation.priority: KeyNavigation.BeforeItem
                KeyNavigation.tab: passwordInput
            }

            Label {
                text: i18n.tr("Password")
            }
            TextField {
                id: passwordInput
                inputMethodHints: Qt.ImhHiddenText | Qt.ImhNoAutoUppercase | Qt.ImhNoPredictiveText
                echoMode: TextInput.Password
                anchors {
                    left: parent.left;
                    right: parent.right;
                }
                KeyNavigation.priority: KeyNavigation.BeforeItem
                KeyNavigation.backtab: userNameInput
            }

            Label {
                id: passwordWarning
                anchors {
                    left: parent.left
                    right: parent.right
                }
                visible: passwordInput.text.length > 0
                wrapMode: TextEdit.Wrap
                text: i18n.tr("This password will be saved in encrypted storage. " +
                          "If you do not enter password here, Dekko will prompt for one when needed.")
            }
            Label {
                id: cancelLabel
                width: parent.width
                wrapMode: TextEdit.Wrap
                text: i18n.tr("Click cancel to enter your details manually")
            }
        }
    }
    Connections {
        target: dekko
        onAutoConfigComplete: {
            closeTimer.start();
        }
        onManualConfig: {
            closeTimer.start();
        }
    }
    Timer {
        id: closeTimer
        interval: 200
        repeat: false
        onTriggered: {
            pageStack.pop();
        }
    }
}
