/* Copyright (C) 2014 Dan Chapman <dpniel@ubuntu.com>

   This file is part of Dekko email client for Ubuntu Devices/

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import QtQuick 2.2
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 1.0 as ListItem
import TrojitaCore 0.1
import DekkoCore 0.1

Page {
    id: generalSettings
    title: i18n.tr("Mail Preferences")
    property Account account
    flickable: null

    property bool autoLoadImages
    property alias preferPlainText: preferPlainTextCheckBox.checked
    property alias autoExpungeOnDelete: expungeCheckBox.checked

    function save() {
        console.log("[LOG] Saving mail preferences.........")
        account.autoLoadImages = autoLoadImages;
        account.preferPlainText = preferPlainText;
        account.imapAutoExpungeOnDelete = autoExpungeOnDelete;
        console.log("[LOG] Preferences saved successfully")
    }
    Flickable {
        id: flicker
        clip: true
        anchors.fill: parent
//        contentHeight: column.height + units.gu(5)

        Column {
            id: column
            anchors {
                margins: units.gu(2)
//                left: parent.left
//                top: parent.top
//                right: parent.right
                fill: parent
            }
            spacing: units.gu(1)

            OptionSelector {
                id: loadImageOptionSelector
                anchors.left: parent.left
                anchors.right: parent.right
                property var autoLoad: {'NEVER': 0, 'ALWAYS': 1}
                text: i18n.tr("Loading Images")
                model: [
                    i18n.tr("Never load images from the Internet"),
                    i18n.tr("Always load images from the internet")
                ]
                onSelectedIndexChanged: {

                    switch (selectedIndex){
                    case 0:
                        autoLoadImages = false
                        break
                    case 1:
                        autoLoadImages = true
                        break
                    }
                }

                Component.onCompleted: {
                    if (account.autoLoadImages) {

                        loadImageOptionSelector.selectedIndex = 1
                    } else {
                        loadImageOptionSelector.selectedIndex = 0
                    }
                }
            }

//            OptionSelector {
//                id: openLinkExternallySelector
//                anchors.left: parent.left
//                anchors.right: parent.right
//                property var autoOpen: {'NEVER': 0, 'ASK': 1, 'ALWAYS': 2}
//                text: i18n.tr("Opening url links in web browser")
//                model: [
//                    i18n.tr("Never open url links"),
//                    i18n.tr("Ask me before opening url links"),
//                    i18n.tr("Always open url links")
//                ]
//                onSelectedIndexChanged: {

//                    switch (selectedIndex){
//                    case autoOpen.NEVER:
////                        imapAccess.autoLoadImages = false
//                        break
//                    case autoOpen.ASK:
//                        break
//                    case autoOpen.ALWAYS:
////                        imapAccess.autoLoadImages = true
//                        break
//                    }
//                }

//                Component.onCompleted: {
//                    if (imapAccess.autoLoadImages) {
//                        loadImageOptionSelector.selectedIndex = autoLoad.ALWAYS
//                    } else {
//                        loadImageOptionSelector.selectedIndex = autoLoad.NEVER
//                    }
//                }
//            }

            Item {
                id: preferPlainTextItem
                width: parent.width
                height: units.gu(4)
                property alias checked: preferPlainTextCheckBox.checked
                Label {
                    text: i18n.tr("Prefer plain text over html messages")
                    anchors.left: parent.left
                    anchors.right: preferPlainTextCheckBox.left
                    anchors.verticalCenter: parent.verticalCenter
                    wrapMode: Text.Wrap
                }
                CheckBox {
                    id: preferPlainTextCheckBox
                    anchors.right: parent.right
                    checked: account.preferPlainText
                }
            }
            Item {
                id: expungeItem
                width: parent.width
                height: units.gu(5)
                property alias checked: expungeCheckBox.checked
                Label {
                    text: i18n.tr("Auto-expunge when a message is marked\nas deleted")
                    anchors.left: parent.left
                    anchors.right: expungeCheckBox.left
                    anchors.verticalCenter: parent.verticalCenter
                    wrapMode: Text.Wrap
                }
                CheckBox {
                    id: expungeCheckBox
                    anchors.right: parent.right
                    checked: account.imapAutoExpungeOnDelete
                }
            }
        }
    }
}
