/* Copyright (C) 2014 Dan Chapman <dpniel@ubuntu.com>

   This file is part of Dekko email client for Ubuntu Devices/

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import QtQuick 2.0
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 1.0 as ListItem
import DekkoCore 0.1
import "../Components"

MultipleSelectionListView {
    id: identityListView
    anchors.fill: parent
    clip: true

    property var _currentSwipedItem: null

    function resetSwipe() {
        if (_currentSwipedItem) {
            _currentSwipedItem.resetSwipe()
            _currentSwipedItem = null
        }
    }

    function _updateSwipeState(item) {
        if (item.swipping) {
            return
        }

        if (item.swipeState !== "Normal") {
            if (_currentSwipedItem !== item) {
                if (_currentSwipedItem) {
                    _currentSwipedItem.resetSwipe()
                }
                _currentSwipedItem = item
            }
        } else if (item.swipeState !== "Normal" && _currentSwipedItem === item) {
            _currentSwipedItem = null
        }
    }

    Connections {
        target: Qt.application
        onActiveChanged: {
            if (!Qt.application.active) {
                identityListView.currentIndex = -1
            }
        }
    }

    currentIndex: -1
    listModel: currentAccount.identityModel

    onSelectionDone: {
        for (var i=0; i < items.count; i++) {
            currentAccount.identityModel.removeRow(items.get(i).index)
        }
    }

    onIsInSelectionModeChanged: {
        if (isInSelectionMode && _currentSwipedItem) {
            _currentSwipedItem.resetSwipe()
            _currentSwipedItem = null
        }
    }
    displaced: Transition {
        UbuntuNumberAnimation {
            property: "y"
        }
    }

    remove: Transition {
        ParallelAnimation {
            UbuntuNumberAnimation {
                property: "height"
                to: 0
            }

            UbuntuNumberAnimation {
                properties: "opacity"
                to: 0
            }
            ScriptAction {
                script: {
                    resetSwipe()
                }
            }
        }
    }

    footer: Item {
        width: parent.width
        height: units.gu(7)
        Button {
            id: addBtn
            width: parent.width - units.gu(2)
            color: UbuntuColors.green
            text: i18n.tr("Add New Identity")
            anchors.verticalCenter: parent.verticalCenter
            anchors.horizontalCenter: parent.horizontalCenter
            onClicked: pageStack.push(Qt.resolvedUrl("NewIdentityPage.qml"))
        }
    }

    listDelegate: SenderIdentityDelegate {
        id: delegate
        formattedIdentity: formattedString
        showDivider: true
        anchors {
            left: parent.left
            right: parent.right
        }
        selected: identityListView.isSelected(delegate)
        selectionMode: identityListView.isInSelectionMode
        isFirst: model.index === 0
        locked: identityListView.isInSelectionMode
        active: ListView.isCurrentItem

        onItemClicked: {
            pageStack.push(editIdentityPage,
                           {
                               name: name,
                               email: email,
                               org: model.organization,
                               sig: signature
                           })
        }

        onSwippingChanged: identityListView._updateSwipeState(delegate)
        onSwipeStateChanged: identityListView._updateSwipeState(delegate)

        leftSideAction: Action {
            text: i18n.tr("Delete")
            iconName: "delete"
            onTriggered: currentAccount.identityModel.removeRow(model.index)
        }

        rightSideActions: []
        NewIdentityPage {
            id: editIdentityPage
            state: "edit"
            onUpdateIdentity: {
                model.name = name
                model.email = email
                model.organization = org
                model.signature = sig
                pageStack.pop()
            }
        }
    }
}
