/* Copyright (C) 2014 Dan Chapman <dpniel@ubuntu.com>

   This file is part of the Dekko

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3 or any later version
   accepted by the membership of KDE e.V. (or its successor approved
   by the membership of KDE e.V.), which shall act as a proxy
   defined in Section 14 of version 3 of the license.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ACCOUNT_H
#define ACCOUNT_H

#include <QObject>
#include <QDebug>
#include <QSettings>
#include "Common/ConnectionMethod.h"
#include "Composer/Submission.h"
#include "Imap/Model/ImapAccess.h"
#include "UiUtils/PasswordWatcher.h"
#include "Plugins/PluginManager.h"
#include "SenderIdentityModel.h"

namespace Dekko
{
namespace Accounts
{
class Account : public QObject
{
    Q_OBJECT
    Q_PROPERTY(QSettings *settings READ settings WRITE setSettings NOTIFY usernameChanged)
    Q_PROPERTY(QObject *identityModel READ identityModel NOTIFY identityModelChanged)
    // ACCOUNT PROPERTIES - these will be used by the accountslistmodel
    Q_PROPERTY(QString accountName READ accountName WRITE setAccountName NOTIFY accountNameChanged)
    Q_PROPERTY(QString description READ description WRITE setDescription NOTIFY descriptionChanged)
    Q_PROPERTY(QString accountImage READ accountImage WRITE setAccountImage NOTIFY accountImageChanged)

    // IMAP SETTINGS
    Q_PROPERTY(QString imapServer READ imapServer WRITE setImapServer NOTIFY serverChanged)
    Q_PROPERTY(int imapPort READ imapPort WRITE setImapPort NOTIFY portChanged)
    Q_PROPERTY(QString imapUsername READ imapUsername WRITE setImapUsername NOTIFY usernameChanged)
    Q_PROPERTY(QString imapSslMode READ imapSslMode WRITE setImapSslMode NOTIFY connMethodChanged)
    Q_PROPERTY(bool imapAutoExpungeOnDelete READ imapAutoExpungeOnDelete WRITE setImapAutoExpungeOnDelete NOTIFY autoExpungeOnDeleteChanged)
    Q_PROPERTY(QStringList imapExtensionBlackList READ imapExtensionBlackList WRITE setImapExtensionBlackList NOTIFY extensionBlackListChanged)
    Q_PROPERTY(QByteArray imapPubKey READ imapPubKey WRITE setImapPubKey NOTIFY pubKeyChanged)
    Q_PROPERTY(bool imapUseSystemProxy READ useSystemProxy WRITE setUseSystemProxy NOTIFY useSystemProxyChanged)
    Q_PROPERTY(QStringList imapProcessArgs READ imapProcessArgs WRITE setImapProcessArgs NOTIFY imapProcessArgsChanged)
    Q_PROPERTY(bool imapEnableId READ imapEnableId WRITE setImapEnableId NOTIFY imapEnableIdChanged)
    Q_PROPERTY(bool imapNeedsNetwork READ imapNeedsNetwork WRITE setImapNeedsNetwork NOTIFY imapNeedsNetworkChanged)
    Q_PROPERTY(bool imapStartOffline READ imapStartOffline WRITE setImapStartOffline NOTIFY imapStartOfflineChanged)
    Q_PROPERTY(QString cacheOffline READ cacheOffline WRITE setCacheOffline NOTIFY cacheOfflineChanged)
    Q_PROPERTY(int cacheOfflineNumberOfDays READ cacheOfflineNumberOfDays WRITE setCacheOfflineNumberOfDays NOTIFY cacheOfflineNumberOfDaysChanged)

    // SUBMISSION SETTINGS
    Q_PROPERTY(QString submissionServer READ submissionServer WRITE setSubmissionServer NOTIFY serverChanged)
    Q_PROPERTY(int submissionPort READ submissionPort WRITE setSubmissionPort NOTIFY portChanged)
    Q_PROPERTY(QString submissionUsername READ submissionUsername WRITE setSubmissionUsername NOTIFY submissionUsernameChanged)
    Q_PROPERTY(bool submissionAuthenticateEnabled READ submissionAuthenticateEnabled WRITE setSubmissionAuthenticateEnabled NOTIFY authenticateEnabledChanged)
    Q_PROPERTY(QString submissionPathToSendmail READ submissionPathToSendmail WRITE setSubmissionPathToSendmail NOTIFY pathToSendmailChanged)
    Q_PROPERTY(bool submissionSaveToImap READ submissionSaveToImap WRITE setSubmissionSaveToImap NOTIFY saveToImapChanged)
    Q_PROPERTY(QString submissionSentMailboxName READ submissionSentMailboxName WRITE setSubmissionSentMailboxName NOTIFY sentMailboxNameChanged)
    Q_PROPERTY(bool submissionUseBurl READ submissionUseBurl WRITE setSubmissionUseBurl NOTIFY useBurlChanged)
    // TODO move MSA::Method to submissionManager
    Q_PROPERTY(Composer::Submission::Method submissionMethod READ submissionMethod WRITE setSubmissionMethod NOTIFY submissionMethodChanged)
    Q_ENUMS(Composer::Submission::Method)

    // PREFERENCES
    Q_PROPERTY(bool autoLoadImages READ autoLoadImages WRITE setAutoLoadImages NOTIFY autoLoadImagesChanged)
    Q_PROPERTY(bool preferPlainText READ preferPlainText WRITE setPreferPlainText NOTIFY preferPlainTextChanged)
    Q_PROPERTY(bool developerModeEnabled READ developerModeEnabled WRITE setDeveloperModeEnabled NOTIFY developerModeEnabledChanged)
    Q_PROPERTY(bool devModeImapModelLogging READ devModeImapModelLogging WRITE setDevModeImapModelLogging NOTIFY devModeImapModelLoggingChanged)
    Q_PROPERTY(bool devModeEnableThreading READ devModeEnableThreading WRITE setDevModeEnableThreading NOTIFY devModeEnableThreadingChanged)
    Q_PROPERTY(bool devModeEnableContacts READ devModeEnableContacts WRITE setDevModeEnableContacts NOTIFY devModeEnableContactsChanged)

    Q_PROPERTY(bool passwordMigrated READ passwordMigrated WRITE setPasswordMigrated NOTIFY passwordMigrationChanged)

public:
    explicit Account(QObject *parent = 0);
    ~Account(){ this->deleteLater(); }

    QSettings *settings();
    void setSettings(QSettings *settings);

    QObject *identityModel() const;
    //ACCOUNT
    QString accountName() const;
    void setAccountName(const QString &name);
    QString description() const;
    void setDescription(const QString &description);
    QString accountImage() const;
    void setAccountImage(const QString &image);

    //IMAP
    QString imapServer() const;
    void setImapServer(const QString &server);
    int imapPort() const;
    void setImapPort(const int &port);
    QString imapUsername() const;
    void setImapUsername(const QString &username);
    QString imapSslMode() const;
    void setImapSslMode(const QString &sslMode);
    bool imapAutoExpungeOnDelete() const;
    void setImapAutoExpungeOnDelete(const bool expunge);
    QStringList imapExtensionBlackList() const;
    void setImapExtensionBlackList(const QStringList &blacklist);
    QByteArray imapPubKey() const;
    void setImapPubKey(const QByteArray pubkey);
    bool useSystemProxy() const;
    void setUseSystemProxy(const bool useProxy);
    QStringList imapProcessArgs() const;
    void setImapProcessArgs(const QStringList &args);
    bool imapEnableId() const;
    void setImapEnableId(const bool enable);
    bool imapNeedsNetwork() const;
    void setImapNeedsNetwork(const bool needsNetwork);
    QString cacheOffline() const;
    void setCacheOffline(const QString &cacheOffline);
    int cacheOfflineNumberOfDays() const;
    void setCacheOfflineNumberOfDays(const int &numDays);
    bool imapStartOffline() const;
    void setImapStartOffline(const bool startOffline);

    Q_INVOKABLE void forgetSslCertificate() const;

    Common::ConnectionMethod imapConnectionMethod() const;
    void setImapConnectionMethod(const Common::ConnectionMethod method);

    // SUBMISSION
    QString submissionServer() const;
    void setSubmissionServer(const QString &server);
    int submissionPort() const;
    void setSubmissionPort(const int &port);
    QString submissionUsername() const;
    void setSubmissionUsername(const QString &username);
    bool submissionAuthenticateEnabled() const;
    void setSubmissionAuthenticateEnabled(const bool authEnabled);
    QString submissionPathToSendmail() const;
    void setSubmissionPathToSendmail(const QString &path);
    bool submissionSaveToImap() const;
    void setSubmissionSaveToImap(const bool save);
    QString submissionSentMailboxName() const;
    void setSubmissionSentMailboxName(const QString &mboxName);
    bool submissionUseBurl() const;
    void setSubmissionUseBurl(const bool useBurl);
    Composer::Submission::Method submissionMethod() const;
    void setSubmissionMethod(const Composer::Submission::Method &method);

    int submissionDefaultPort(const Composer::Submission::Method &method);

    // PREFERENCES
    bool autoLoadImages() const;
    void setAutoLoadImages(const bool autoLoad);
    bool preferPlainText() const;
    void setPreferPlainText(const bool preferPlainText);
    bool developerModeEnabled() const;
    void setDeveloperModeEnabled(const bool enabled);
    bool devModeImapModelLogging() const;
    void setDevModeImapModelLogging(const bool enabled);
    bool devModeEnableThreading() const;
    void setDevModeEnableThreading(const bool enabled);
    bool devModeEnableContacts() const;
    void setDevModeEnableContacts(const bool enabled);

    bool passwordMigrated() const;
    void setPasswordMigrated(const bool migrated);

    QString generateNewAccountId() const;

signals:
    void saved();
    void savingFailed();
    void reloadSucceeded();
    void reloadFailed();
    void accountSettingsDeleted();

    // ACCOUNT
    void accountNameChanged();
    void descriptionChanged();
    void accountImageChanged();

    // IMAP
    void serverChanged();
    void portChanged();
    void usernameChanged();
    void connMethodChanged();
    void checkSslPolicy();
    void autoExpungeOnDeleteChanged();
    void extensionBlackListChanged();
    void pubKeyChanged();
    void useSystemProxyChanged();
    void imapProcessArgsChanged();
    void imapEnableIdChanged();
    void imapNeedsNetworkChanged();
    void cacheOfflineChanged();
    void cacheOfflineNumberOfDaysChanged();
    void imapStartOfflineChanged();

    // SUBMISSION
    void authenticateEnabledChanged();
    void pathToSendmailChanged();
    void saveToImapChanged();
    void sentMailboxNameChanged();
    void useBurlChanged();
    void submissionMethodChanged();
    void showPortWarning(QString warning);
    void submissionUsernameChanged();

    // PREFERENCES
    void autoLoadImagesChanged();
    void preferPlainTextChanged();
    void developerModeEnabledChanged();
    void devModeImapModelLoggingChanged();
    void devModeEnableThreadingChanged();
    void devModeEnableContactsChanged();

    // TEMPORARY IDENTITY
    void identityNameChanged();
    void identityAddressChanged();
    void identityOrgChanged();
    void identitySignatureChanged();
    void useDefaultSignatureChanged();

    void passwordMigrationChanged();
    void identityModelChanged();

public slots:
    void reloadSettings(); // reload settings for current account name
    void reloadSettings(const QString &accountName);
    void reloadSettings(QSettings *settings, const QString &accountName); // reload settings for given accountName
    void reset();
    void save();
    void save(QSettings *settings); // save settings to file
    void deleteAccountSettings(); // delete account settings
    void nukeCurrentCache();
    void maybeShowPortWarning();
    void appendExtensionToImapBlacklist(const QString &extension);
    void removeExtensionFromBlackList(const QString &extension);

private:
    QSettings *m_settings;
    SenderIdentityModel *m_identityModel;
    Common::ConnectionMethod m_imapConnectionMethod;

    // ACCOUNT
    QString m_accountName;
    QString m_description;
    QString m_accountImagePath;

    // IMAP
    QString m_imapServer;
    int m_imapPort;
    QString m_imapUserName;
    QString m_imapSslMode;
    bool m_imapAutoExpunge;
    QStringList m_imapExtBlacklist;
    QByteArray m_imapPubKey;
    bool m_useSysProxy;
    QStringList m_imapProcessArgs;
    bool m_imapEnableId;
    bool m_imapNeedsNetwork;
    QString m_cacheOffline;
    int m_cacheOfflineNumberOfDays;
    bool m_imapStartOffline;

    // SUBMISSION
    QString m_submissionServer;
    int m_submissionPort;
    QString m_submissionUsername;
    bool m_submissionAuthEnabled;
    Composer::Submission::Method m_msaSubmissionMethod;
    QString m_submissionPathToSendmail;
    bool m_submissionSaveToImap;
    QString m_submissionSentMailboxName;
    bool m_submissionUseBurl;

    // PREFERENCES
    bool m_prefsAutoLoadImages;
    bool m_prefsPreferPlainText;
    bool m_devModeEnable;
    bool m_devModeImapLogging;
    bool m_devModeEnableThreading;
    bool m_devModeEnableContacts;

    // TEMPORARY IDENTITY
    QString m_identityName;
    QString m_identityAddress;
    QString m_identityOrg;
    QString m_identitySignature;
    bool m_useDefaultSignature;

    bool m_pwdMigrated;

    QString getDefaultSignature() const;

};
}
}
#endif // ACCOUNT_H
